﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyShip
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    internal class DrawningLiner : DrawningShip
    {
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес корабля</param>
        /// <param name="bodyColor">Цвет палубы</param>
        /// <param name="dopColor">Дополнительный цвет</param>
        /// <param name="swimmingPool">Признак наличия бассейна</param>
        /// <param name="dopDeck">Признак наличия дополнительной палубы</param>
        public DrawningLiner(int speed, float weight, Color bodyColor, Color dopColor, bool swimmingPool, bool dopDeck) :
            base(speed, weight, bodyColor, 110, 60)
        {
            Ship = new EntityLiner(speed, weight, bodyColor, dopColor, swimmingPool, dopDeck);
        }
        public override void DrawTransport(Graphics g)
        {
            if (Ship is not EntityLiner liner)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush dopBrush = new SolidBrush(liner.DopColor);
            
            if (liner.DopDeck)
            {
                g.FillRectangle(dopBrush, _startPosX + 45, _startPosY, 35, 15);
                g.DrawRectangle(pen, _startPosX + 45, _startPosY, 35, 15);
            }
           
            _startPosY += 10;
            base.DrawTransport(g);
            _startPosY -= 10;

            if (liner.SwimmingPool)
            {
                g.FillRectangle(dopBrush, _startPosX + 25, _startPosY + 15, 15, 7);
                g.DrawRectangle(pen, _startPosX + 25, _startPosY + 15, 15, 7);
            }

        }
    }
}
